@@ -1,7 +1,7 @@
 use super::{Handle, ManagedTypeApi, ManagedTypeApiImpl};
 use crate::types::{
     heap::{Address, Box, H256},
-    BigUint, EsdtLocalRoleFlags, EsdtTokenData, ManagedAddress, ManagedByteArray, TokenIdentifier,
+    EsdtLocalRoleFlags, EsdtTokenData, ManagedAddress, TokenIdentifier,
 };
 
 pub trait BlockchainApi: ManagedTypeApi {
@@ -19,20 +19,20 @@ pub trait BlockchainApi: ManagedTypeApi {
 pub trait BlockchainApiImpl: ManagedTypeApiImpl {
     fn get_caller_legacy(&self) -> Address;
 
-    fn get_caller_handle(&self) -> Handle {
-        self.mb_new_from_bytes(self.get_caller_legacy().as_bytes())
+    fn load_caller_managed(&self, dest: Handle) {
+        self.mb_overwrite(dest, self.get_caller_legacy().as_bytes());
     }
 
     fn get_sc_address_legacy(&self) -> Address;
 
-    fn get_sc_address_handle(&self) -> Handle {
-        self.mb_new_from_bytes(self.get_sc_address_legacy().as_bytes())
+    fn load_sc_address_managed(&self, dest: Handle) {
+        self.mb_overwrite(dest, self.get_sc_address_legacy().as_bytes())
     }
 
     fn get_owner_address_legacy(&self) -> Address;
 
-    fn get_owner_address_handle(&self) -> Handle {
-        self.mb_new_from_bytes(self.get_owner_address_legacy().as_bytes())
+    fn load_owner_address_managed(&self, dest: Handle) {
+        self.mb_overwrite(dest, self.get_owner_address_legacy().as_bytes())
     }
 
     fn get_shard_of_address_legacy(&self, address: &Address) -> u32;
@@ -51,25 +51,24 @@ pub trait BlockchainApiImpl: ManagedTypeApiImpl {
         self.is_smart_contract_legacy(&address)
     }
 
-    fn get_balance_legacy(&self, address: &Address) -> Handle;
+    fn load_balance_legacy(&self, dest: Handle, address: &Address);
 
-    fn get_balance_handle(&self, address_handle: Handle) -> Handle {
+    fn load_balance(&self, dest: Handle, address_handle: Handle) {
         let mut address = Address::zero();
         let _ = self.mb_load_slice(address_handle, 0, address.as_mut());
-        self.get_balance_legacy(&address)
+        self.load_balance_legacy(dest, &address);
     }
 
     fn get_state_root_hash_legacy(&self) -> H256;
 
-    #[inline]
-    fn get_state_root_hash<M: ManagedTypeApi>(&self) -> ManagedByteArray<M, 32> {
-        ManagedByteArray::new_from_bytes(self.get_state_root_hash_legacy().as_array())
+    fn load_state_root_hash_managed(&self, dest: Handle) {
+        self.mb_overwrite(dest, self.get_state_root_hash_legacy().as_bytes());
     }
 
     fn get_tx_hash_legacy(&self) -> H256;
 
-    fn get_tx_hash<M: ManagedTypeApi>(&self) -> ManagedByteArray<M, 32> {
-        ManagedByteArray::new_from_bytes(self.get_tx_hash_legacy().as_array())
+    fn load_tx_hash_managed(&self, dest: Handle) {
+        self.mb_overwrite(dest, self.get_tx_hash_legacy().as_bytes());
     }
 
     fn get_gas_left(&self) -> u64;
@@ -84,8 +83,8 @@ pub trait BlockchainApiImpl: ManagedTypeApiImpl {
 
     fn get_block_random_seed_legacy(&self) -> Box<[u8; 48]>;
 
-    fn get_block_random_seed<M: ManagedTypeApi>(&self) -> ManagedByteArray<M, 48> {
-        ManagedByteArray::new_from_bytes(&*self.get_block_random_seed_legacy())
+    fn load_block_random_seed_managed(&self, dest: Handle) {
+        self.mb_overwrite(dest, &*self.get_block_random_seed_legacy());
     }
 
     fn get_prev_block_timestamp(&self) -> u64;
@@ -98,22 +97,19 @@ pub trait BlockchainApiImpl: ManagedTypeApiImpl {
 
     fn get_prev_block_random_seed_legacy(&self) -> Box<[u8; 48]>;
 
-    fn get_prev_block_random_seed<M: ManagedTypeApi>(&self) -> ManagedByteArray<M, 48> {
-        ManagedByteArray::new_from_bytes(&*self.get_prev_block_random_seed_legacy())
+    fn load_prev_block_random_seed_managed(&self, dest: Handle) {
+        self.mb_overwrite(dest, &*self.get_prev_block_random_seed_legacy());
     }
 
-    fn get_current_esdt_nft_nonce<M: ManagedTypeApi>(
-        &self,
-        address: &ManagedAddress<M>,
-        token_id: &TokenIdentifier<M>,
-    ) -> u64;
+    fn get_current_esdt_nft_nonce(&self, address_handle: Handle, token_id_handle: Handle) -> u64;
 
-    fn get_esdt_balance<M: ManagedTypeApi>(
+    fn load_esdt_balance(
         &self,
-        address: &ManagedAddress<M>,
-        token_id: &TokenIdentifier<M>,
+        address_handle: Handle,
+        token_id_handle: Handle,
         nonce: u64,
-    ) -> BigUint<M>;
+        dest: Handle,
+    );
 
     fn get_esdt_token_data<M: ManagedTypeApi>(
         &self,
@@ -122,8 +118,16 @@ pub trait BlockchainApiImpl: ManagedTypeApiImpl {
         nonce: u64,
     ) -> EsdtTokenData<M>;
 
-    fn get_esdt_local_roles<M: ManagedTypeApi>(
+    #[deprecated(
+        since = "0.31.1",
+        note = "Only used for ;imited backwards compatibility tests. Never use! Use `get_esdt_token_data` instead."
+    )]
+    fn get_esdt_token_data_unmanaged<M: ManagedTypeApi>(
         &self,
+        address: &ManagedAddress<M>,
         token_id: &TokenIdentifier<M>,
-    ) -> EsdtLocalRoleFlags;
+        nonce: u64,
+    ) -> EsdtTokenData<M>;
+
+    fn get_esdt_local_roles(&self, token_id_handle: Handle) -> EsdtLocalRoleFlags;
 }
