@@ -1,6 +1,6 @@
 use crate::types::{heap::BoxedBytes, MessageHashType};
 
-use super::{ManagedTypeApi, ManagedTypeApiImpl};
+use super::{Handle, ManagedTypeApi, ManagedTypeApiImpl};
 
 pub const SHA256_RESULT_LEN: usize = 32;
 pub const KECCAK256_RESULT_LEN: usize = 32;
@@ -17,13 +17,11 @@ pub trait CryptoApi: ManagedTypeApi {
 pub trait CryptoApiImpl: ManagedTypeApiImpl {
     fn sha256_legacy(&self, data: &[u8]) -> [u8; SHA256_RESULT_LEN];
 
-    #[cfg(feature = "ei-1-1")]
-    fn sha256(&self, data_handle: super::Handle) -> super::Handle;
+    fn sha256_managed(&self, dest: Handle, data_handle: Handle);
 
     fn keccak256_legacy(&self, data: &[u8]) -> [u8; KECCAK256_RESULT_LEN];
 
-    #[cfg(feature = "ei-1-1")]
-    fn keccak256(&self, data_handle: super::Handle) -> super::Handle;
+    fn keccak256_managed(&self, dest: Handle, data_handle: Handle);
 
     fn ripemd160(&self, data: &[u8]) -> [u8; RIPEMD_RESULT_LEN];
 
