@@ -7,13 +7,13 @@ use super::{
 pub type Handle = i32;
 
 pub trait ManagedTypeApiImpl: BigIntApi + EllipticCurveApi + ManagedBufferApi + ErrorApi {
-    fn mb_to_big_int_unsigned(&self, buffer_handle: Handle) -> Handle;
+    fn mb_to_big_int_unsigned(&self, buffer_handle: Handle, dest: Handle);
 
-    fn mb_to_big_int_signed(&self, buffer_handle: Handle) -> Handle;
+    fn mb_to_big_int_signed(&self, buffer_handle: Handle, dest: Handle);
 
-    fn mb_from_big_int_unsigned(&self, big_int_handle: Handle) -> Handle;
+    fn mb_from_big_int_unsigned(&self, big_int_handle: Handle, dest: Handle);
 
-    fn mb_from_big_int_signed(&self, big_int_handle: Handle) -> Handle;
+    fn mb_from_big_int_signed(&self, big_int_handle: Handle, dest: Handle);
 
     fn validate_token_identifier(&self, token_id_handle: Handle) -> bool {
         let token_id_len = self.mb_len(token_id_handle);
