@@ -15,18 +15,15 @@ pub trait StorageReadApiImpl {
 
     fn storage_load_to_heap(&self, key: &[u8]) -> Box<[u8]>;
 
-    fn storage_load_big_uint_raw(&self, key: &[u8]) -> Handle;
+    fn storage_load_big_uint_raw(&self, key: &[u8], dest: Handle);
 
-    fn storage_load_managed_buffer_raw(&self, key_handle: Handle) -> Handle;
-
-    fn storage_load_managed_buffer_len(&self, key_handle: Handle) -> usize;
+    fn storage_load_managed_buffer_raw(&self, key_handle: Handle, dest: Handle);
 
     fn storage_load_u64(&self, key: &[u8]) -> u64;
 
     fn storage_load_i64(&self, key: &[u8]) -> i64;
 
-    #[cfg(feature = "ei-1-1")]
-    fn storage_load_from_address(&self, address_handle: Handle, key_handle: Handle) -> Handle;
+    fn storage_load_from_address(&self, address_handle: Handle, key_handle: Handle, dest: Handle);
 }
 
 pub trait StorageWriteApi {
