@@ -11,19 +11,19 @@ impl ManagedTypeApi for UncallableApi {
 }
 
 impl ManagedTypeApiImpl for UncallableApi {
-    fn mb_to_big_int_unsigned(&self, _buffer_handle: Handle) -> Handle {
+    fn mb_to_big_int_unsigned(&self, _buffer_handle: Handle, _dest: Handle) {
         unreachable!()
     }
 
-    fn mb_to_big_int_signed(&self, _buffer_handle: Handle) -> Handle {
+    fn mb_to_big_int_signed(&self, _buffer_handle: Handle, _dest: Handle) {
         unreachable!()
     }
 
-    fn mb_from_big_int_unsigned(&self, _big_int_handle: Handle) -> Handle {
+    fn mb_from_big_int_unsigned(&self, _big_int_handle: Handle, _dest: Handle) {
         unreachable!()
     }
 
-    fn mb_from_big_int_signed(&self, _big_int_handle: Handle) -> Handle {
+    fn mb_from_big_int_signed(&self, _big_int_handle: Handle, _dest: Handle) {
         unreachable!()
     }
 }
