@@ -1,6 +1,5 @@
 use super::{
-    BlockchainWrapper, CallValueWrapper, CryptoWrapper, ErrorHelper, ManagedSerializer,
-    PrintHelper, SendWrapper,
+    BlockchainWrapper, CallValueWrapper, CryptoWrapper, ErrorHelper, ManagedSerializer, SendWrapper,
 };
 use crate::api::VMApi;
 
@@ -49,9 +48,4 @@ pub trait ContractBase: Sized {
     fn error(&self) -> ErrorHelper<Self::Api> {
         ErrorHelper::new_instance()
     }
-
-    #[inline]
-    fn print(&self) -> PrintHelper<Self::Api> {
-        PrintHelper::new()
-    }
 }
