@@ -2,8 +2,8 @@ use core::marker::PhantomData;
 
 use crate::{
     api::{
-        BlockchainApi, BlockchainApiImpl, ErrorApi, ErrorApiImpl, ManagedTypeApi, StorageReadApi,
-        StorageReadApiImpl,
+        BlockchainApi, BlockchainApiImpl, ErrorApi, ErrorApiImpl, ManagedTypeApi, StaticVarApiImpl,
+        StorageReadApi, StorageReadApiImpl,
     },
     storage::{self},
     types::{
@@ -44,7 +44,9 @@ where
 
     #[inline]
     pub fn get_caller(&self) -> ManagedAddress<A> {
-        ManagedAddress::from_raw_handle(A::blockchain_api_impl().get_caller_handle())
+        let handle = A::static_var_api_impl().next_handle();
+        A::blockchain_api_impl().load_caller_managed(handle);
+        ManagedAddress::from_raw_handle(handle)
     }
 
     #[cfg(feature = "alloc")]
@@ -55,7 +57,9 @@ where
 
     #[inline]
     pub fn get_sc_address(&self) -> ManagedAddress<A> {
-        ManagedAddress::from_raw_handle(A::blockchain_api_impl().get_sc_address_handle())
+        let handle = A::static_var_api_impl().next_handle();
+        A::blockchain_api_impl().load_sc_address_managed(handle);
+        ManagedAddress::from_raw_handle(handle)
     }
 
     #[cfg(feature = "alloc")]
@@ -66,7 +70,9 @@ where
 
     #[inline]
     pub fn get_owner_address(&self) -> ManagedAddress<A> {
-        ManagedAddress::from_raw_handle(A::blockchain_api_impl().get_owner_address_handle())
+        let handle = A::static_var_api_impl().next_handle();
+        A::blockchain_api_impl().load_owner_address_managed(handle);
+        ManagedAddress::from_raw_handle(handle)
     }
 
     pub fn check_caller_is_owner(&self) {
@@ -100,14 +106,16 @@ where
     #[cfg(feature = "alloc")]
     #[inline]
     pub fn get_balance_legacy(&self, address: &crate::types::Address) -> BigUint<A> {
-        BigUint::from_raw_handle(A::blockchain_api_impl().get_balance_legacy(address))
+        let handle = A::static_var_api_impl().next_handle();
+        A::blockchain_api_impl().load_balance_legacy(handle, address);
+        BigUint::from_raw_handle(handle)
     }
 
     #[inline]
     pub fn get_balance(&self, address: &ManagedAddress<A>) -> BigUint<A> {
-        BigUint::from_raw_handle(
-            A::blockchain_api_impl().get_balance_handle(address.get_raw_handle()),
-        )
+        let handle = A::static_var_api_impl().next_handle();
+        A::blockchain_api_impl().load_balance(handle, address.get_raw_handle());
+        BigUint::from_raw_handle(handle)
     }
 
     #[inline]
@@ -127,7 +135,9 @@ where
 
     #[inline]
     pub fn get_state_root_hash(&self) -> ManagedByteArray<A, 32> {
-        A::blockchain_api_impl().get_state_root_hash()
+        let handle = A::static_var_api_impl().next_handle();
+        A::blockchain_api_impl().load_state_root_hash_managed(handle);
+        ManagedByteArray::from_raw_handle(handle)
     }
 
     #[cfg(feature = "alloc")]
@@ -138,7 +148,9 @@ where
 
     #[inline]
     pub fn get_tx_hash(&self) -> ManagedByteArray<A, 32> {
-        A::blockchain_api_impl().get_tx_hash::<A>()
+        let handle = A::static_var_api_impl().next_handle();
+        A::blockchain_api_impl().load_tx_hash_managed(handle);
+        ManagedByteArray::from_raw_handle(handle)
     }
 
     #[inline]
@@ -174,7 +186,9 @@ where
 
     #[inline]
     pub fn get_block_random_seed(&self) -> ManagedByteArray<A, 48> {
-        A::blockchain_api_impl().get_block_random_seed::<A>()
+        let handle = A::static_var_api_impl().next_handle();
+        A::blockchain_api_impl().load_block_random_seed_managed(handle);
+        ManagedByteArray::from_raw_handle(handle)
     }
 
     #[inline]
@@ -205,7 +219,9 @@ where
 
     #[inline]
     pub fn get_prev_block_random_seed(&self) -> ManagedByteArray<A, 48> {
-        A::blockchain_api_impl().get_prev_block_random_seed::<A>()
+        let handle = A::static_var_api_impl().next_handle();
+        A::blockchain_api_impl().load_prev_block_random_seed_managed(handle);
+        ManagedByteArray::from_raw_handle(handle)
     }
 
     #[inline]
@@ -214,7 +230,8 @@ where
         address: &ManagedAddress<A>,
         token_id: &TokenIdentifier<A>,
     ) -> u64 {
-        A::blockchain_api_impl().get_current_esdt_nft_nonce::<A>(address, token_id)
+        A::blockchain_api_impl()
+            .get_current_esdt_nft_nonce(address.get_raw_handle(), token_id.get_raw_handle())
     }
 
     #[inline]
@@ -224,7 +241,14 @@ where
         token_id: &TokenIdentifier<A>,
         nonce: u64,
     ) -> BigUint<A> {
-        A::blockchain_api_impl().get_esdt_balance::<A>(address, token_id, nonce)
+        let result_handle = A::static_var_api_impl().next_handle();
+        A::blockchain_api_impl().load_esdt_balance(
+            address.get_raw_handle(),
+            token_id.get_raw_handle(),
+            nonce,
+            result_handle,
+        );
+        BigUint::from_raw_handle(result_handle)
     }
 
     #[inline]
@@ -237,8 +261,9 @@ where
         A::blockchain_api_impl().get_esdt_token_data::<A>(address, token_id, nonce)
     }
 
+    #[inline]
     pub fn get_esdt_local_roles(&self, token_id: &TokenIdentifier<A>) -> EsdtLocalRoleFlags {
-        A::blockchain_api_impl().get_esdt_local_roles::<A>(token_id)
+        A::blockchain_api_impl().get_esdt_local_roles(token_id.get_raw_handle())
     }
 }
 
@@ -249,8 +274,9 @@ where
     /// Retrieves validator rewards, as set by the protocol.
     #[inline]
     pub fn get_cumulated_validator_rewards(&self) -> BigUint<A> {
-        let raw_handle = A::storage_read_api_impl()
-            .storage_load_big_uint_raw(storage::protected_keys::ELROND_REWARD_KEY);
-        BigUint::from_raw_handle(raw_handle)
+        let result_handle = A::static_var_api_impl().next_handle();
+        A::storage_read_api_impl()
+            .storage_load_big_uint_raw(storage::protected_keys::ELROND_REWARD_KEY, result_handle);
+        BigUint::from_raw_handle(result_handle)
     }
 }
