@@ -2,7 +2,6 @@ mod blockchain_wrapper;
 mod call_value_wrapper;
 mod crypto_wrapper;
 mod error_helper;
-mod print_helper;
 mod send_wrapper;
 mod serializer;
 
@@ -10,6 +9,5 @@ pub use blockchain_wrapper::BlockchainWrapper;
 pub use call_value_wrapper::CallValueWrapper;
 pub use crypto_wrapper::CryptoWrapper;
 pub use error_helper::ErrorHelper;
-pub use print_helper::PrintHelper;
 pub use send_wrapper::SendWrapper;
 pub use serializer::{ExitCodecErrorHandler, ManagedSerializer};
