@@ -25,6 +25,7 @@ pub const STORAGE_VALUE_EXCEEDS_BUFFER: &[u8] = b"storage value exceeds buffer";
 pub const FINISH_ENCODE_ERROR: &[u8] = b"endpoint result encode error: ";
 pub const SERIALIZER_DECODE_ERROR: &[u8] = b"serializer decode error: ";
 pub const SERIALIZER_ENCODE_ERROR: &[u8] = b"serializer encode error: ";
+pub const FORMATTER_ENCODE_ERROR: &[u8] = b"formatter encode error: ";
 pub const LOG_TOPIC_ENCODE_ERROR: &[u8] = b"log topic encode error: ";
 pub const LOG_DATA_ENCODE_ERROR: &[u8] = b"log data encode error: ";
 pub const CONTRACT_CALL_ENCODE_ERROR: &[u8] = b"contract call encode error: ";
@@ -43,6 +44,8 @@ pub const DESERIALIZATION_ARG_OUT_OF_RANGE: &str =
 
 pub const CALLBACK_BAD_FUNC: &[u8] = b"no callback function with that name exists in contract";
 
+pub const RECIPIENT_ADDRESS_NOT_SET: &[u8] = b"recipient address not set";
+
 pub const STORAGE_NOT_I64: &[u8] = b"storage not i64";
 pub const STORAGE_NOT_32_BYTES: &[u8] = b"32 bytes of data expected in storage at key";
 
