@@ -1,5 +1,5 @@
 use super::SEPARATOR;
-use crate::{err_msg, hex_util::hex_digits_to_byte};
+use crate::{err_msg, formatter::hex_util::hex_digits_to_byte};
 use alloc::{boxed::Box, vec::Vec};
 use elrond_codec::{DecodeError, DecodeErrorHandler, TopDecodeMultiInput};
 
