@@ -25,7 +25,6 @@ pub mod esdt;
 pub mod external_view_contract;
 pub mod formatter;
 pub mod hex_call_data;
-pub mod hex_util;
 pub mod io;
 pub mod log_util;
 mod macros;
@@ -35,5 +34,4 @@ pub mod types;
 
 pub use hex_call_data::*;
 pub use hex_literal;
-pub use io::*;
 pub use storage::{storage_clear, storage_get, storage_get_len, storage_set};
