@@ -24,7 +24,7 @@ macro_rules! imports {
             io::*,
             non_zero_usize,
             non_zero_util::*,
-            require, require_old, sc_error, sc_panic, sc_print,
+            require, require_old, sc_error, sc_format, sc_panic, sc_print,
             storage::mappers::*,
             types::{
                 SCResult::{Err, Ok},
@@ -113,7 +113,7 @@ macro_rules! sc_panic {
 /// }
 ///
 /// fn only_accept_negative(&self, x: i32) {
-///     require!(x < 0, "only negative values accepted, {:x} is not negative", x);
+///     require!(x < 0, "only negative values accepted, {} is not negative", x);
 /// }
 ///
 /// fn only_accept_zero(&self, x: i32, message: &ManagedBuffer<Self::Api>) {
@@ -134,9 +134,25 @@ macro_rules! require {
 macro_rules! sc_print {
     ($msg:tt, $($arg:expr),* $(,)?) => {{
         let mut ___buffer___ =
-            elrond_wasm::types::ManagedBufferCachedBuilder::<Self::Api>::new_from_slice(&[]);
+            <<Self::Api as elrond_wasm::api::PrintApi>::PrintApiImpl as elrond_wasm::api::PrintApiImpl>::Buffer::default();
         elrond_wasm::derive::format_receiver_args!(___buffer___, $msg, $($arg),*);
-        <Self::Api as elrond_wasm::api::PrintApi>::print_api_impl().print_managed_buffer(___buffer___.into_managed_buffer().get_raw_handle());
+        <<Self::Api as elrond_wasm::api::PrintApi>::PrintApiImpl as elrond_wasm::api::PrintApiImpl>::print_buffer(
+            &<Self::Api as elrond_wasm::api::PrintApi>::print_api_impl(),
+            ___buffer___,
+        );
+    }};
+}
+
+#[macro_export]
+macro_rules! sc_format {
+    ($msg:tt, $($arg:expr),+ $(,)?) => {{
+        let mut ___buffer___ =
+            elrond_wasm::types::ManagedBufferCachedBuilder::<Self::Api>::new_from_slice(&[]);
+        elrond_wasm::derive::format_receiver_args!(___buffer___, $msg, $($arg),+);
+        ___buffer___.into_managed_buffer()
+    }};
+    ($msg:expr $(,)?) => {{
+        elrond_wasm::types::ManagedBuffer::new_from_bytes($msg.as_bytes())
     }};
 }
 
