@@ -1,3 +1,4 @@
+mod bi_di_mapper;
 mod fungible_token_mapper;
 mod linked_list_mapper;
 mod map_mapper;
@@ -14,6 +15,7 @@ mod user_mapper;
 mod vec_mapper;
 mod whitelist_mapper;
 
+pub use bi_di_mapper::BiDiMapper;
 pub use fungible_token_mapper::FungibleTokenMapper;
 pub use linked_list_mapper::{LinkedListMapper, LinkedListNode};
 pub use map_mapper::MapMapper;
@@ -22,7 +24,7 @@ pub use mapper::{StorageClearable, StorageMapper};
 pub use non_fungible_token_mapper::NonFungibleTokenMapper;
 pub use queue_mapper::QueueMapper;
 pub use set_mapper::SetMapper;
-pub use single_value_mapper::SingleValueMapper;
+pub use single_value_mapper::{SingleValue, SingleValueMapper};
 pub use token_attributes_mapper::TokenAttributesMapper;
 pub use token_mapper::StorageTokenWrapper;
 pub use unordered_set_mapper::UnorderedSetMapper;
