@@ -1,7 +1,16 @@
-use crate::abi::{TypeAbi, TypeName};
+use crate::{
+    abi::{TypeAbi, TypeName},
+    formatter::{hex_util, FormatByteReceiver, SCBinary, SCDisplay, SCLowerHex},
+};
 use bitflags::bitflags;
 use elrond_codec::*;
 
+const UPGRADEABLE_STRING: &[u8] = b"Upgradeable";
+const READABLE_STRING: &[u8] = b"Readable";
+const PAYABLE_STRING: &[u8] = b"Payable";
+const PAYABLE_BY_SC_STRING: &[u8] = b"PayableBySC";
+const DEFAULT_STRING: &[u8] = b"Default";
+
 bitflags! {
     #[derive(Default)]
     pub struct CodeMetadata: u16 {
@@ -98,6 +107,60 @@ impl TypeAbi for CodeMetadata {
     }
 }
 
+impl SCDisplay for CodeMetadata {
+    fn fmt<F: FormatByteReceiver>(&self, f: &mut F) {
+        let mut nothing_printed: bool = true;
+        verify_metadata_and_append(
+            self.is_upgradeable(),
+            f,
+            UPGRADEABLE_STRING,
+            &mut nothing_printed,
+        );
+        verify_metadata_and_append(self.is_readable(), f, READABLE_STRING, &mut nothing_printed);
+        verify_metadata_and_append(self.is_payable(), f, PAYABLE_STRING, &mut nothing_printed);
+        verify_metadata_and_append(
+            self.is_payable_by_sc(),
+            f,
+            PAYABLE_BY_SC_STRING,
+            &mut nothing_printed,
+        );
+        if nothing_printed {
+            f.append_bytes(DEFAULT_STRING);
+        }
+    }
+}
+
+impl SCLowerHex for CodeMetadata {
+    fn fmt<F: FormatByteReceiver>(&self, f: &mut F) {
+        let num = self.bits().to_be_bytes();
+        f.append_bytes(&hex_util::byte_to_hex_digits(num[0])[..]);
+        f.append_bytes(&hex_util::byte_to_hex_digits(num[1])[..]);
+    }
+}
+
+impl SCBinary for CodeMetadata {
+    fn fmt<F: FormatByteReceiver>(&self, f: &mut F) {
+        let num = self.bits().to_be_bytes();
+        f.append_bytes(&hex_util::byte_to_binary_digits(num[0])[..]);
+        f.append_bytes(&hex_util::byte_to_binary_digits(num[1])[..]);
+    }
+}
+
+fn verify_metadata_and_append<F: FormatByteReceiver>(
+    constraint: bool,
+    f: &mut F,
+    bytes_to_append: &[u8],
+    nothing_printed: &mut bool,
+) {
+    if constraint {
+        if !*nothing_printed {
+            f.append_bytes(b"|");
+        }
+        f.append_bytes(bytes_to_append);
+        *nothing_printed = false;
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
