@@ -2,7 +2,7 @@ use crate::{
     api::{ErrorApi, Handle, ManagedTypeApi},
     contract_base::ExitCodecErrorHandler,
     err_msg,
-    types::{ManagedBuffer, ManagedType, ManagedVec, ManagedVecRefIterator},
+    types::{ManagedBuffer, ManagedType, ManagedVec, ManagedVecRefIterator, MultiValueEncoded},
 };
 use alloc::vec::Vec;
 use elrond_codec::{
@@ -107,6 +107,10 @@ where
         }
         v
     }
+
+    pub fn into_multi_value_encoded(self) -> MultiValueEncoded<M, ManagedBuffer<M>> {
+        self.data.into()
+    }
 }
 
 impl<M> ManagedArgBuffer<M>
