@@ -1,4 +1,4 @@
-use elrond_codec::{TopDecodeMulti, TopEncodeMulti};
+use elrond_codec::{CodecFrom, TopEncodeMulti};
 
 use crate::{
     api::{
@@ -7,11 +7,11 @@ use crate::{
     },
     contract_base::{BlockchainWrapper, ExitCodecErrorHandler},
     err_msg,
+    io::{ArgErrorHandler, ArgId, ManagedResultArgLoader},
     types::{
         AsyncCall, BigUint, EsdtTokenPayment, ManagedAddress, ManagedArgBuffer, ManagedBuffer,
-        ManagedType, ManagedVec, TokenIdentifier,
+        ManagedVec, TokenIdentifier,
     },
-    ArgErrorHandler, ArgId, ManagedResultArgLoader,
 };
 use core::marker::PhantomData;
 
@@ -26,36 +26,39 @@ const TRANSFER_EXECUTE_DEFAULT_LEFTOVER: u64 = 100_000;
 /// Represents metadata for calling another contract.
 /// Can transform into either an async call, transfer call or other types of calls.
 #[must_use]
-pub struct ContractCall<SA, R>
+pub struct ContractCall<SA, OriginalResult>
 where
     SA: CallTypeApi + 'static,
 {
     _phantom: PhantomData<SA>,
-    to: ManagedAddress<SA>,
-    egld_payment: BigUint<SA>,
-    payments: ManagedVec<SA, EsdtTokenPayment<SA>>,
-    endpoint_name: ManagedBuffer<SA>,
-    explicit_gas_limit: u64,
-    arg_buffer: ManagedArgBuffer<SA>,
-    _return_type: PhantomData<R>,
+    pub to: ManagedAddress<SA>,
+    pub egld_payment: BigUint<SA>,
+    pub payments: ManagedVec<SA, EsdtTokenPayment<SA>>,
+    pub endpoint_name: ManagedBuffer<SA>,
+    pub extra_gas_for_callback: u64,
+    pub explicit_gas_limit: u64,
+    pub arg_buffer: ManagedArgBuffer<SA>,
+    pub success_callback: &'static [u8],
+    pub error_callback: &'static [u8],
+    _return_type: PhantomData<OriginalResult>,
 }
 
 /// Syntactical sugar to help macros to generate code easier.
-/// Unlike calling `ContractCall::<SA, R>::new`, here types can be inferred from the context.
-pub fn new_contract_call<SA, R>(
+/// Unlike calling `ContractCall::<SA, OriginalResult>::new`, here types can be inferred from the context.
+pub fn new_contract_call<SA, OriginalResult>(
     to: ManagedAddress<SA>,
     endpoint_name_slice: &'static [u8],
     payments: ManagedVec<SA, EsdtTokenPayment<SA>>,
-) -> ContractCall<SA, R>
+) -> ContractCall<SA, OriginalResult>
 where
     SA: CallTypeApi + 'static,
 {
     let endpoint_name = ManagedBuffer::new_from_bytes(endpoint_name_slice);
-    ContractCall::<SA, R>::new_with_esdt_payment(to, endpoint_name, payments)
+    ContractCall::<SA, OriginalResult>::new_with_esdt_payment(to, endpoint_name, payments)
 }
 
 #[allow(clippy::return_self_not_must_use)]
-impl<SA, R> ContractCall<SA, R>
+impl<SA, OriginalResult> ContractCall<SA, OriginalResult>
 where
     SA: CallTypeApi + 'static,
 {
@@ -71,15 +74,20 @@ where
     ) -> Self {
         let arg_buffer = ManagedArgBuffer::new_empty();
         let egld_payment = BigUint::zero();
+        let success_callback = b"";
+        let error_callback = b"";
         ContractCall {
             _phantom: PhantomData,
             to,
             egld_payment,
             payments,
             explicit_gas_limit: UNSPECIFIED_GAS_LIMIT,
+            extra_gas_for_callback: UNSPECIFIED_GAS_LIMIT,
             endpoint_name,
             arg_buffer,
             _return_type: PhantomData,
+            success_callback,
+            error_callback,
         }
     }
 
@@ -116,6 +124,27 @@ where
         self
     }
 
+    #[cfg(feature = "promises")]
+    #[inline]
+    pub fn with_success_callback(mut self, callback: &'static [u8]) -> Self {
+        self.success_callback = callback;
+        self
+    }
+
+    #[cfg(feature = "promises")]
+    #[inline]
+    pub fn with_error_callback(mut self, callback: &'static [u8]) -> Self {
+        self.error_callback = callback;
+        self
+    }
+
+    #[cfg(feature = "promises")]
+    #[inline]
+    pub fn with_extra_gas_for_callback(mut self, gas_limit: u64) -> Self {
+        self.extra_gas_for_callback = gas_limit;
+        self
+    }
+
     #[inline]
     pub fn with_gas_limit(mut self, gas_limit: u64) -> Self {
         self.explicit_gas_limit = gas_limit;
@@ -171,7 +200,9 @@ where
                 let mut new_arg_buffer = ManagedArgBuffer::new_empty();
                 new_arg_buffer.push_arg(&payment.token_identifier);
                 new_arg_buffer.push_arg(&payment.amount);
-                new_arg_buffer.push_arg(&self.endpoint_name);
+                if !self.endpoint_name.is_empty() {
+                    new_arg_buffer.push_arg(&self.endpoint_name);
+                }
 
                 let zero = BigUint::zero();
                 let endpoint_name = ManagedBuffer::new_from_bytes(ESDT_TRANSFER_FUNC_NAME);
@@ -182,9 +213,12 @@ where
                     egld_payment: zero,
                     payments: no_payments,
                     explicit_gas_limit: self.explicit_gas_limit,
+                    extra_gas_for_callback: self.extra_gas_for_callback,
                     endpoint_name,
                     arg_buffer: new_arg_buffer.concat(self.arg_buffer),
                     _return_type: PhantomData,
+                    success_callback: self.success_callback,
+                    error_callback: self.error_callback,
                 }
             } else {
                 let payments = self.no_payments();
@@ -200,12 +234,12 @@ where
                 new_arg_buffer.push_arg(&payment.token_nonce);
                 new_arg_buffer.push_arg(&payment.amount);
                 new_arg_buffer.push_arg(&self.to);
-                new_arg_buffer.push_arg(&self.endpoint_name);
+                if !self.endpoint_name.is_empty() {
+                    new_arg_buffer.push_arg(&self.endpoint_name);
+                }
 
                 // nft transfer is sent to self, sender = receiver
-                let recipient_addr = ManagedAddress::from_raw_handle(
-                    SA::blockchain_api_impl().get_sc_address_handle(),
-                );
+                let recipient_addr = BlockchainWrapper::<SA>::new().get_sc_address();
                 let zero = BigUint::zero();
                 let endpoint_name = ManagedBuffer::new_from_bytes(ESDT_NFT_TRANSFER_FUNC_NAME);
 
@@ -215,9 +249,12 @@ where
                     egld_payment: zero,
                     payments,
                     explicit_gas_limit: self.explicit_gas_limit,
+                    extra_gas_for_callback: self.extra_gas_for_callback,
                     endpoint_name,
                     arg_buffer: new_arg_buffer.concat(self.arg_buffer),
                     _return_type: PhantomData,
+                    success_callback: self.success_callback,
+                    error_callback: self.error_callback,
                 }
             }
         } else {
@@ -238,7 +275,9 @@ where
             new_arg_buffer.push_arg(payment.token_nonce);
             new_arg_buffer.push_arg(payment.amount);
         }
-        new_arg_buffer.push_arg(self.endpoint_name);
+        if !self.endpoint_name.is_empty() {
+            new_arg_buffer.push_arg(self.endpoint_name);
+        }
 
         // multi transfer is sent to self, sender = receiver
         let recipient_addr = BlockchainWrapper::<SA>::new().get_sc_address();
@@ -251,13 +290,16 @@ where
             egld_payment: zero,
             payments,
             explicit_gas_limit: self.explicit_gas_limit,
+            extra_gas_for_callback: self.extra_gas_for_callback,
             endpoint_name,
             arg_buffer: new_arg_buffer.concat(self.arg_buffer),
             _return_type: PhantomData,
+            success_callback: self.success_callback,
+            error_callback: self.error_callback,
         }
     }
 
-    fn resolve_gas_limit(&self) -> u64 {
+    pub fn resolve_gas_limit(&self) -> u64 {
         if self.explicit_gas_limit == UNSPECIFIED_GAS_LIMIT {
             SA::blockchain_api_impl().get_gas_left()
         } else {
@@ -276,24 +318,47 @@ where
             callback_call: None,
         }
     }
+
+    #[cfg(feature = "promises")]
+    pub fn register_promise(mut self) {
+        self = self.convert_to_esdt_transfer_call();
+        SA::send_api_impl().create_async_call_raw(
+            &self.to,
+            &self.egld_payment,
+            &self.endpoint_name,
+            self.success_callback,
+            self.error_callback,
+            self.explicit_gas_limit,
+            self.extra_gas_for_callback,
+            &self.arg_buffer,
+        )
+    }
 }
 
-impl<SA, R> ContractCall<SA, R>
+impl<SA, OriginalResult> ContractCall<SA, OriginalResult>
 where
     SA: CallTypeApi + 'static,
-    R: TopDecodeMulti,
+    OriginalResult: TopEncodeMulti,
 {
-    fn decode_result(raw_result: ManagedVec<SA, ManagedBuffer<SA>>) -> R {
+    fn decode_result<RequestedResult>(
+        raw_result: ManagedVec<SA, ManagedBuffer<SA>>,
+    ) -> RequestedResult
+    where
+        RequestedResult: CodecFrom<OriginalResult>,
+    {
         let mut loader = ManagedResultArgLoader::new(raw_result);
         let arg_id = ArgId::from(&b"sync result"[..]);
         let h = ArgErrorHandler::<SA>::from(arg_id);
-        let Ok(result) = R::multi_decode_or_handle_err(&mut loader, h);
+        let Ok(result) = RequestedResult::multi_decode_or_handle_err(&mut loader, h);
         result
     }
 
     /// Executes immediately, synchronously, and returns contract call result.
     /// Only works if the target contract is in the same shard.
-    pub fn execute_on_dest_context(mut self) -> R {
+    pub fn execute_on_dest_context<RequestedResult>(mut self) -> RequestedResult
+    where
+        RequestedResult: CodecFrom<OriginalResult>,
+    {
         self = self.convert_to_esdt_transfer_call();
         let raw_result = SA::send_api_impl().execute_on_dest_context_raw(
             self.resolve_gas_limit(),
@@ -303,34 +368,15 @@ where
             &self.arg_buffer,
         );
 
+        SA::send_api_impl().clean_return_data();
+
         Self::decode_result(raw_result)
     }
 
-    /// Executes immediately, synchronously, and returns contract call result.
-    /// Only works if the target contract is in the same shard.
-    /// This is a workaround to handle nested sync calls.
-    /// Please do not use this method unless there is absolutely no other option.
-    /// Will be eliminated after some future Arwen hook redesign.
-    /// `range_closure` takes the number of results before, the number of results after,
-    /// and is expected to return the start index (inclusive) and end index (exclusive).
-    pub fn execute_on_dest_context_custom_range<F>(mut self, range_closure: F) -> R
+    pub fn execute_on_dest_context_readonly<RequestedResult>(mut self) -> RequestedResult
     where
-        F: FnOnce(usize, usize) -> (usize, usize),
+        RequestedResult: CodecFrom<OriginalResult>,
     {
-        self = self.convert_to_esdt_transfer_call();
-        let raw_result = SA::send_api_impl().execute_on_dest_context_raw_custom_result_range(
-            self.resolve_gas_limit(),
-            &self.to,
-            &self.egld_payment,
-            &self.endpoint_name,
-            &self.arg_buffer,
-            range_closure,
-        );
-
-        Self::decode_result(raw_result)
-    }
-
-    pub fn execute_on_dest_context_readonly(mut self) -> R {
         self = self.convert_to_esdt_transfer_call();
         let raw_result = SA::send_api_impl().execute_on_dest_context_readonly_raw(
             self.resolve_gas_limit(),
@@ -339,16 +385,20 @@ where
             &self.arg_buffer,
         );
 
+        SA::send_api_impl().clean_return_data();
+
         Self::decode_result(raw_result)
     }
 }
 
-impl<SA, R> ContractCall<SA, R>
+impl<SA, OriginalResult> ContractCall<SA, OriginalResult>
 where
     SA: CallTypeApi + 'static,
 {
     /// Executes immediately, synchronously.
+    ///
     /// The result (if any) is ignored.
+    ///
     /// Only works if the target contract is in the same shard.
     pub fn execute_on_dest_context_ignore_result(mut self) {
         self = self.convert_to_esdt_transfer_call();
@@ -359,6 +409,8 @@ where
             &self.endpoint_name,
             &self.arg_buffer,
         );
+
+        SA::send_api_impl().clean_return_data();
     }
 
     pub fn execute_on_same_context(mut self) {
@@ -370,6 +422,8 @@ where
             &self.endpoint_name,
             &self.arg_buffer,
         );
+
+        SA::send_api_impl().clean_return_data();
     }
 
     fn resolve_gas_limit_with_leftover(&self) -> u64 {
@@ -385,6 +439,7 @@ where
     }
 
     /// Immediately launches a transfer-execute call.
+    ///
     /// This is similar to an async call, but there is no callback
     /// and there can be more than one such call per transaction.
     pub fn transfer_execute(self) {
