@@ -1,4 +1,3 @@
-mod async_call_result;
 mod codec_multi_value_aliases;
 mod operation_completion_status;
 mod sc_error;
@@ -6,7 +5,6 @@ mod sc_error_managed;
 mod sc_error_static;
 mod sc_result;
 
-pub use async_call_result::{AsyncCallError, AsyncCallResult};
 pub use codec_multi_value_aliases::*;
 pub use operation_completion_status::OperationCompletionStatus;
 pub use sc_error::SCError;
