@@ -1,6 +1,6 @@
 use elrond_codec::{
     multi_types::MultiValue3, DecodeErrorHandler, EncodeErrorHandler, TopDecodeMulti,
-    TopDecodeMultiInput, TopEncodeMulti, TopEncodeMultiOutput,
+    TopDecodeMultiInput, TopDecodeMultiLength, TopEncodeMulti, TopEncodeMultiOutput,
 };
 
 use crate::{
@@ -63,8 +63,6 @@ impl<M> TopEncodeMulti for EsdtTokenPaymentMultiValue<M>
 where
     M: ManagedTypeApi,
 {
-    type DecodeAs = Self;
-
     fn multi_encode_or_handle_err<O, H>(&self, output: &mut O, h: H) -> Result<(), H::HandledErr>
     where
         O: TopEncodeMultiOutput,
@@ -93,6 +91,13 @@ where
     }
 }
 
+impl<M> TopDecodeMultiLength for EsdtTokenPaymentMultiValue<M>
+where
+    M: ManagedTypeApi,
+{
+    const LEN: usize = 3;
+}
+
 impl<M> TypeAbi for EsdtTokenPaymentMultiValue<M>
 where
     M: ManagedTypeApi,
