@@ -3,7 +3,7 @@ use core::convert::TryFrom;
 use crate::{
     abi::{TypeAbi, TypeName},
     api::{Handle, ManagedTypeApi},
-    hex_util::encode_bytes_as_hex,
+    formatter::{hex_util::encode_bytes_as_hex, FormatByteReceiver, SCLowerHex},
     types::{ManagedBuffer, ManagedType},
 };
 use elrond_codec::{
@@ -212,6 +212,15 @@ where
     }
 }
 
+impl<M, const N: usize> SCLowerHex for ManagedByteArray<M, N>
+where
+    M: ManagedTypeApi,
+{
+    fn fmt<F: FormatByteReceiver>(&self, f: &mut F) {
+        SCLowerHex::fmt(&self.buffer, f)
+    }
+}
+
 impl<M, const N: usize> core::fmt::Debug for ManagedByteArray<M, N>
 where
     M: ManagedTypeApi,
